"""Nbconvert preprocessor for the python-markdown nbextension."""

import base64
import os
import re

from ipython_genutils.ipstruct import Struct
from nbconvert.preprocessors import Preprocessor
from traitlets import Bool, Unicode

try:
    from urllib.request import urlopen  # py3
except ImportError:
    from urllib2 import urlopen


class EmbedImagesPreprocessor(Preprocessor):
    """
    :mod:`nbconvert` Preprocessor to embed images in a markdown cell as
        attachment inside the notebook itself.

    This :class:`~nbconvert.preprocessors.Preprocessor` replaces kernel code in
    markdown cells with the results stored in the cell metadata.

    The preprocessor is installed by default. To enable embedding images with
    NbConvert, you need to set the configuration parameter
    `EmbedImagesPreprocessor.embed_images=True`.
    This can be done either in the `jupyter_nbconvert_config.py` file::

        c.EmbedImagesPreprocessor.embed_images=True

    or using a command line parameter when calling NbConvert::

        $ jupyter nbconvert --to html --EmbedImagesPreprocessor.embed_images=True mynotebook.ipynb

    Further options are::

        EmbedImagesPreprocessor.embed_remote_images=True

    to additionally embeds all images referenced by an url
    (e.g. http://jupyter.org/assets/nav_logo.svg) instead of a local file name.

    Another configuration option is::

        EmbedImagesPreprocessor.resize=small

    Let's you scale-down the size of an image. This is useful if you want to
    save space by not embedding large images and instead use a smaller (scaled)
    version. Works only for raster images (i.e. png, jpg).
    Valid resize settings are: small = 500px, mid = 1000px, large = 2000px
    for maximum size in length or width.  No upscaling of small images will
    be performed. The Python package `PIL` needs to be installed for this
    option to work.

    Example::

            $ jupyter nbconvert --to html --EmbedImagesPreprocessor.embed_images=True
                --EmbedImagesPreprocessor.resize=large mynotebook.ipynb

    *Note:* To embed images after conversion to HTML you can also use the
           `html_embed` exporter
    """

    embed_images = Bool(False, help="Embed images as attachment").tag(config=True)
    embed_remote_images = Bool(False, help="Embed images referenced by an url as attachment").tag(config=True)
    resize = Unicode('', help="Resize images to save space (reduce size)").tag(config=True)
    imgsizes = {'small': 500, 'mid': 1000, 'large': 2000}

    def preprocess(self, nb, resources):
        """Skip preprocessor if not enabled"""
        if self.embed_images:
            nb, resources = super(EmbedImagesPreprocessor, self).preprocess(nb, resources)
        return nb, resources

    def resize_image(self, imgname, imgformat, imgdata):
        """Resize images if desired and PIL is installed

        Parameters
        ----------
            imgname: str
                Name of image
            imgformat: str
                Format of image (JPG or PNG)
            imgdata:
                Binary image data

        """
        if imgformat in ['png', 'jpg']:
            from io import BytesIO
            try:
                from PIL import Image
            except ImportError:
                self.log.info("Pillow library not available to resize images")
                return imgdata
            # Only make images smaller when rescaling
            im = Image.open(BytesIO(imgdata))
            factor = self.imgsizes[self.resize] / max(im.size)
            if factor < 1.0:
                newsize = (int(im.size[0] * factor), int(im.size[1] * factor))
                newim = im.resize(newsize)
                fp = BytesIO()
                # PIL requires JPEG instead of JPG
                newim.save(fp, format=imgformat.replace('jpg', 'jpeg'))
                imgdata = fp.getvalue()
                fp.close()
                self.log.debug("Resized %d x %d image %s to size %d x %d pixels" %
                               (im.size[0], im.size[1], imgname, newsize[0], newsize[1]))
        return imgdata

    def replfunc_md(self, match):
        """Read image and store as base64 encoded attachment"""
        url = match.group(2)
        imgformat = url.split('.')[-1].lower()
        if url.startswith('http'):
            if self.embed_remote_images:
                data = urlopen(url).read()
            else:
                return match.group(0)
        elif url.startswith('attachment'):
            return match.group(0)
        else:
            filename = os.path.join(self.path, url)
            with open(filename, 'rb') as f:
                data = f.read()

        if self.resize in self.imgsizes.keys():
            data = self.resize_image(url, imgformat, data)

        self.log.debug("Embedding url: %s, format: %s" % (url, imgformat))
        b64_data = base64.b64encode(data).decode("utf-8")
        self.attachments[url] = {'image/' + imgformat: b64_data}

        newimg = '![' + match.group(1) + '](attachment:' + match.group(2) + ')'
        return newimg

    def preprocess_cell(self, cell, resources, index):
        """
        Preprocess cell

        Parameters
        ----------
        cell : NotebookNode cell
            Notebook cell being processed
        resources : dictionary
            Additional resources used in the conversion process.  Allows
            preprocessors to pass variables into the Jinja engine.
        index : int
            Index of the cell being processed (see base.py)
        """
        self.path = resources['metadata']['path']
        self.attachments = getattr(cell, 'attachments', Struct())

        if cell.cell_type == "markdown":
            regex = re.compile('!\[([^"]*)\]\(([^"]+)\)')
            cell.source = regex.sub(self.replfunc_md, cell.source)
            cell.attachments = self.attachments
        return cell, resources



# -*- coding: utf-8 -*-
"""Nbconvert preprocessor for the python-markdown nbextension."""

import re

from nbconvert.preprocessors import Preprocessor


class PyMarkdownPreprocessor(Preprocessor):
    """
    :mod:`nbconvert` Preprocessor for the python-markdown nbextension.

    This :class:`~nbconvert.preprocessors.Preprocessor` replaces kernel code in
    markdown cells with the results stored in the cell metadata.
    """

    def replace_variables(self, source, variables):
        """
        Replace {{variablename}} with stored value
        """
        try:
            replaced = re.sub(
                "{{(.*?)}}", lambda m: variables.get(m.group(1), ''), source)
        except TypeError:
            replaced = source
        return replaced

    def preprocess_cell(self, cell, resources, index):
        """
        Preprocess cell

        Parameters
        ----------
        cell : NotebookNode cell
            Notebook cell being processed
        resources : dictionary
            Additional resources used in the conversion process.  Allows
            preprocessors to pass variables into the Jinja engine.
        cell_index : int
            Index of the cell being processed (see base.py)
        """
        if cell.cell_type == "markdown":
            if hasattr(cell['metadata'], 'variables'):
                variables = cell['metadata']['variables']
                if len(variables) > 0:
                    cell.source = self.replace_variables(
                        cell.source, variables)
        return cell, resources



print("import: 'jupyter_contrib_nbextensions'")
import jupyter_contrib_nbextensions

print("import: 'jupyter_contrib_nbextensions.application'")
import jupyter_contrib_nbextensions.application

print("import: 'jupyter_contrib_nbextensions.install'")
import jupyter_contrib_nbextensions.install

print("import: 'jupyter_contrib_nbextensions.migrate'")
import jupyter_contrib_nbextensions.migrate

print("import: 'jupyter_contrib_nbextensions.nbconvert_support'")
import jupyter_contrib_nbextensions.nbconvert_support

print("import: 'jupyter_contrib_nbextensions.nbconvert_support.collapsible_headings'")
import jupyter_contrib_nbextensions.nbconvert_support.collapsible_headings

print("import: 'jupyter_contrib_nbextensions.nbconvert_support.embedhtml'")
import jupyter_contrib_nbextensions.nbconvert_support.embedhtml

print("import: 'jupyter_contrib_nbextensions.nbconvert_support.pp_highlighter'")
import jupyter_contrib_nbextensions.nbconvert_support.pp_highlighter

print("import: 'jupyter_contrib_nbextensions.nbconvert_support.pre_codefolding'")
import jupyter_contrib_nbextensions.nbconvert_support.pre_codefolding

print("import: 'jupyter_contrib_nbextensions.nbconvert_support.pre_pymarkdown'")
import jupyter_contrib_nbextensions.nbconvert_support.pre_pymarkdown

print("import: 'jupyter_contrib_nbextensions.nbconvert_support.pre_svg2pdf'")
import jupyter_contrib_nbextensions.nbconvert_support.pre_svg2pdf

print("import: 'jupyter_contrib_nbextensions.nbconvert_support.toc2'")
import jupyter_contrib_nbextensions.nbconvert_support.toc2




# -*- coding: utf-8 -*-

"""nbTranslate Preprocessor Exporter class"""

# -----------------------------------------------------------------------------
# Copyright (c) 2016, J.-F. Bercher
#
# Distributed under the terms of the Modified BSD License.
#
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Imports
# -----------------------------------------------------------------------------

from __future__ import print_function

import nbformat
from nbconvert.exporters.notebook import NotebookExporter
from nbconvert.preprocessors import Preprocessor
from traitlets import Bool, Enum, Unicode

# -----------------------------------------------------------------------------
# Preprocessor
# -----------------------------------------------------------------------------

langs = {
    'auto': 'Automatic',
    'af': 'Afrikaans',
    'sq': 'Albanian',
    'ar': 'Arabic',
    'hy': 'Armenian',
    'az': 'Azerbaijani',
    'eu': 'Basque',
    'be': 'Belarusian',
    'bn': 'Bengali',
    'bs': 'Bosnian',
    'bg': 'Bulgarian',
    'ca': 'Catalan',
    'ceb': 'Cebuano',
    'ny': 'Chichewa',
    'zh-cn': 'Chinese Simplified',
    'zh-tw': 'Chinese Traditional',
    'co': 'Corsican',
    'hr': 'Croatian',
    'cs': 'Czech',
    'da': 'Danish',
    'nl': 'Dutch',
    'en': 'English',
    'eo': 'Esperanto',
    'et': 'Estonian',
    'tl': 'Filipino',
    'fi': 'Finnish',
    'fr': 'French',
    'fy': 'Frisian',
    'gl': 'Galician',
    'ka': 'Georgian',
    'de': 'German',
    'el': 'Greek',
    'gu': 'Gujarati',
    'ht': 'Haitian Creole',
    'ha': 'Hausa',
    'haw': 'Hawaiian',
    'iw': 'Hebrew',
    'hi': 'Hindi',
    'hmn': 'Hmong',
    'hu': 'Hungarian',
    'is': 'Icelandic',
    'ig': 'Igbo',
    'id': 'Indonesian',
    'ga': 'Irish',
    'it': 'Italian',
    'ja': 'Japanese',
    'jw': 'Javanese',
    'kn': 'Kannada',
    'kk': 'Kazakh',
    'km': 'Khmer',
    'ko': 'Korean',
    'ku': 'Kurdish (Kurmanji)',
    'ky': 'Kyrgyz',
    'lo': 'Lao',
    'la': 'Latin',
    'lv': 'Latvian',
    'lt': 'Lithuanian',
    'lb': 'Luxembourgish',
    'mk': 'Macedonian',
    'mg': 'Malagasy',
    'ms': 'Malay',
    'ml': 'Malayalam',
    'mt': 'Maltese',
    'mi': 'Maori',
    'mr': 'Marathi',
    'mn': 'Mongolian',
    'my': 'Myanmar (Burmese)',
    'ne': 'Nepali',
    'no': 'Norwegian',
    'ps': 'Pashto',
    'fa': 'Persian',
    'pl': 'Polish',
    'pt': 'Portuguese',
    'ma': 'Punjabi',
    'ro': 'Romanian',
    'ru': 'Russian',
    'sm': 'Samoan',
    'gd': 'Scots Gaelic',
    'sr': 'Serbian',
    'st': 'Sesotho',
    'sn': 'Shona',
    'sd': 'Sindhi',
    'si': 'Sinhala',
    'sk': 'Slovak',
    'sl': 'Slovenian',
    'so': 'Somali',
    'es': 'Spanish',
    'su': 'Sudanese',
    'sw': 'Swahili',
    'sv': 'Swedish',
    'tg': 'Tajik',
    'ta': 'Tamil',
    'te': 'Telugu',
    'th': 'Thai',
    'tr': 'Turkish',
    'uk': 'Ukrainian',
    'ur': 'Urdu',
    'uz': 'Uzbek',
    'vi': 'Vietnamese',
    'cy': 'Welsh',
    'xh': 'Xhosa',
    'yi': 'Yiddish',
    'yo': 'Yoruba',
    'zu': 'Zulu'
}


class nbTranslatePreprocessor(Preprocessor):

    def __init__(self, lang='en', **kw):
        self.language = lang

    def __call__(self, nb, resources, lang='en'):
        if self.enabled:
            self.log.debug("Applying preprocessor: %s",
                           self.__class__.__name__)
            return self.preprocess(nb, resources)
        else:
            return nb, resources

    def preprocess(self, nb, resources):
        """
        Preprocessing to apply on each notebook.

        Must return modified nb, resources.

        If you wish to apply your preprocessing to each cell, you might want
        to override preprocess_cell method instead.

        Parameters
        ----------
        nb : NotebookNode
            Notebook being converted
        resources : dictionary
            Additional resources used in the conversion process.  Allows
            preprocessors to pass variables into the Jinja engine.
        """

        filtered_cells = []
        for cell in nb.cells:
            if cell.cell_type == 'markdown':
                if (cell.get('metadata', {}).get('lang', self.language) == self.language): # noqa
                    filtered_cells.append(cell)
            else:
                filtered_cells.append(cell)

        nb.cells = filtered_cells
        return super(nbTranslatePreprocessor, self).preprocess(nb, resources)

    def preprocess_cell(self, cell, resources, index):
        """
        Preprocess cell

        Parameters
        ----------
        cell : NotebookNode cell
            Notebook cell being processed
        resources : dictionary
            Additional resources used in the conversion process.  Allows
            preprocessors to pass variables into the Jinja engine.
        cell_index : int
            Index of the cell being processed (see base.py)
        """

        return cell, resources

# ----------------------------------------------------------------
# Exporter
# ----------------------------------------------------------------


class NotebookLangExporter(NotebookExporter):
    """Exports to an IPython notebook."""

    nbformat_version = Enum(list(nbformat.versions),
                            default_value=nbformat.current_nbformat,
                            config=True,
                            help="""The nbformat version to write.
        Use this to downgrade notebooks.
        """
                            )

#    language = CaselessStrEnum(
#        langs.keys(), help="Selected language").tag(config=True, alias="lang")

    language = Unicode(
        'en', help="Selected language").tag(config=True, alias="lang")

    addSuffix = Bool(True, help="Use language tag as suffix")

    # language = 'en'

    def _file_extension_default(self):
        return '.ipynb'

    output_mimetype = 'application/json'

    def from_notebook_node(self, nb, resources=None, **kw):

        if (self.language not in langs.keys()):
            raise ValueError("""Error -- {} is not a valid language abbreviation
            Please select one of the abbreviations in the list\n {}""".format(self.language, langs)) # noqa

        nbtranslatepreprocessor = nbTranslatePreprocessor(lang=self.language)
        self.register_preprocessor(nbtranslatepreprocessor, enabled=True)
        self._init_preprocessors()
        nb, resources = nbtranslatepreprocessor(nb, resources)

        nb_copy, resources = super(NotebookLangExporter, self).from_notebook_node(nb, resources, **kw) # noqa
        if self.addSuffix:
            resources['output_suffix'] = '_' + self.language

        return nb_copy, resources



"""Classes and functions provided for use with nbconvert."""

import os

from .collapsible_headings import ExporterCollapsibleHeadings
from .embedhtml import EmbedHTMLExporter
from .execute_time import ExecuteTimePreprocessor
from .exporter_inliner import ExporterInliner
from .nbTranslate import NotebookLangExporter
from .pp_highlighter import HighlighterPostProcessor, HighlighterPreprocessor
from .pre_codefolding import CodeFoldingPreprocessor
from .pre_embedimages import EmbedImagesPreprocessor
from .pre_pymarkdown import PyMarkdownPreprocessor
from .pre_svg2pdf import SVG2PDFPreprocessor
from .toc2 import TocExporter

__all__ = [
    'templates_directory',
    'CodeFoldingPreprocessor',
    'EmbedHTMLExporter',
    'ExecuteTimePreprocessor',
    'ExporterCollapsibleHeadings',
    'ExporterInliner',
    'HighlighterPostProcessor',
    'HighlighterPreprocessor',
    'EmbedImagesPreprocessor',
    'NotebookLangExporter',
    'PyMarkdownPreprocessor',
    'SVG2PDFPreprocessor',
    'TocExporter',
]


def templates_directory():
    """Return path to the jupyter_contrib_nbextensions nbconvert templates."""
    return os.path.join(
        os.path.dirname(os.path.dirname(__file__)), 'templates')



"""
Module containing a preprocessor tto execute code cells, updating time metadata
"""

from datetime import datetime

from nbconvert.preprocessors.execute import ExecutePreprocessor

try:
    # notebook >= 5.0.0-rc1
    import notebook._tz as nbtz
except ImportError:
    # notebook < 5.0.0-rc1
    import notebook.services.contents.tz as nbtz


class ExecuteTimePreprocessor(ExecutePreprocessor):
    """
    Executes all the cells in a notebook, updating their ExecuteTime metadata.
    """

    def run_cell(self, cell, cell_index, *args, **kwargs):
        before = datetime.utcnow()
        exec_reply, outs = super(ExecuteTimePreprocessor, self).run_cell(
            cell, cell_index, *args, **kwargs)

        if exec_reply.get('msg_type', '') == 'execute_reply':
            ets = cell.setdefault('metadata', {}).setdefault('ExecuteTime', {})
            if 'started' in exec_reply.get('metadata', {}):
                # started value should is already a string, so don't isoformat
                ets['start_time'] = exec_reply['metadata']['started']
            else:
                # attempt to fallback to datetime obj for execution request msg
                ets['start_time'] = exec_reply.get(
                    'parent_header', {}).get('date', before).isoformat()
            ets['end_time'] = (
                exec_reply.get('header', {}).get('date') or nbtz.utcnow()
            ).isoformat()

        return exec_reply, outs



# -*- coding: utf-8 -*-
"""App to install/remove jupyter_contrib_nbextensions."""

from __future__ import print_function, unicode_literals

import copy
import sys

from jupyter_contrib_core.notebook_compat.nbextensions import ArgumentConflict
from jupyter_core.application import JupyterApp
from tornado.log import LogFormatter
from traitlets import Bool, Unicode, default

import jupyter_contrib_nbextensions
from jupyter_contrib_nbextensions.install import (
    NotebookRunningError, install, toggle_install_config, toggle_install_files,
    uninstall,
)
from jupyter_contrib_nbextensions.migrate import migrate


class BaseContribNbextensionsApp(JupyterApp):
    """Base class for jupyter_contrib_nbextensions apps."""

    version = jupyter_contrib_nbextensions.__version__

    _log_formatter_cls = LogFormatter

    @default('log_datefmt')
    def _log_datefmt_default(self):
        """Exclude date from timestamp."""
        return "%H:%M:%S"

    @default('log_format')
    def _log_format_default(self):
        """Override default log format to include time."""
        return ('%(color)s['
                '%(levelname)1.1s %(asctime)s %(name)s'
                ']%(end_color)s '
                '%(message)s')


class BaseContribNbextensionsInstallApp(BaseContribNbextensionsApp):
    """Install/Uninstall jupyter_contrib_nbextensions."""

    aliases = {
        'prefix': 'BaseContribNbextensionsInstallApp.prefix',
        'nbextensions': 'BaseContribNbextensionsInstallApp.nbextensions_dir',
    }
    flags = {
        'debug': JupyterApp.flags['debug'],
        'user': ({
            'BaseContribNbextensionsInstallApp': {'user': True}},
            'Perform the operation for the current user'
        ),
        'system': ({
            'BaseContribNbextensionsInstallApp': {
                'user': False, 'sys_prefix': False}},
            'Perform the operation system-wide'
        ),
        'sys-prefix': (
            {'BaseContribNbextensionsInstallApp': {'sys_prefix': True}},
            'Use sys.prefix as the prefix for installing'
        ),
        # below flags apply only to nbextensions, not server extensions
        'overwrite': (
            {'BaseContribNbextensionsInstallApp': {'overwrite': True}},
            'Force overwrite of existing nbextension files, '
            'regardless of modification time'
        ),
        'symlink': (
            {'BaseContribNbextensionsInstallApp': {'symlink': True}},
            'Create symlinks for nbextensions instead of copying files'
        ),
        'skip-running-check': (
            {'BaseContribNbextensionsInstallApp':
                {'skip_running_check': True}},
            'Perform actions even if notebook server(s) are already running'
        ),
        'perform-running-check': (
            {'BaseContribNbextensionsInstallApp':
                {'skip_running_check': False}},
            'Only perform actions if no notebook server(s) are running'
        ),
    }

    _conflicting_flagsets = [['--user', '--system', '--sys-prefix'], ]

    user = Bool(False, config=True, help='Whether to do a user install')
    sys_prefix = Bool(False, config=True,
                      help='Use the sys.prefix as the prefix')

    # settings pertaining to nbextensions installation only
    overwrite = Bool(False, config=True,
                     help='Force overwrite of existing nbextension files')
    symlink = Bool(False, config=True,
                   help='Create symlinks instead of copying nbextension files')
    prefix = Unicode(
        '', config=True,
        help='Installation prefix, currently only used for nbextensions')
    nbextensions_dir = Unicode(
        '', config=True,
        help='Full path to nbextensions dir '
        '(consider instead using system, sys_prefix, prefix or user option)')
    skip_running_check = Bool(
        True, config=True,
        help='Perform actions even if notebook server(s) are already running')

    def parse_command_line(self, argv=None):
        """
        Overriden to check for conflicting flags

        Since notebook version doesn't do it very well
        """
        for conflicting_flags in map(set, self._conflicting_flagsets):
            if len(conflicting_flags.intersection(set(argv))) > 1:
                raise ArgumentConflict(
                    'cannot specify more than one of {}'.format(
                        ', '.join(conflicting_flags)))
        return super(BaseContribNbextensionsInstallApp,
                     self).parse_command_line(argv)


BaseContribNbextensionsInstallApp.flags['s'] = (
    BaseContribNbextensionsInstallApp.flags['symlink'])


class InstallContribNbextensionsApp(BaseContribNbextensionsInstallApp):
    """Install jupyter_contrib_nbextensions."""

    _toggle_value = True  # whether to install or uninstall

    flags = copy.deepcopy(BaseContribNbextensionsInstallApp.flags)
    flags.update({
        'only-config': (
            {'BaseContribNbextensionsInstallApp': {'only_config': True}},
            'Edit config files, but do not install/remove nbextensions files'
        ),
        'only-files': (
            {'BaseContribNbextensionsInstallApp': {'only_files': True}},
            'Install/remove nbextensions files, but do not edit config files'
        ),
    })

    _conflicting_flagsets = (
        BaseContribNbextensionsInstallApp._conflicting_flagsets +
        ['--only-config', '--only-files'])

    only_config = Bool(False, config=True, help=(
        'Edit config files, but do not install/remove nbextensions files'))
    only_files = Bool(False, config=True, help=(
        'Install/remove nbextensions files, but do not edit config files'))

    @property
    def name(self):
        return 'jupyter contrib nbextension {}'.format(
            'install' if self._toggle_value else 'uninstall')

    @property
    def description(self):
        return '{} jupyter_contrib_nbextensions.'.format(
            'Install' if self._toggle_value else 'Uninstall')

    def start(self):
        """Perform the App's actions as configured."""
        if self.extra_args:
            sys.exit('{} takes no extra arguments'.format(self.name))
        self.log.info('{} {}'.format(self.name, ' '.join(self.argv)))
        kwargs = dict(
            user=self.user, sys_prefix=self.sys_prefix, logger=self.log,
            skip_running_check=self.skip_running_check)
        kwargs_files = dict(**kwargs)
        kwargs_files.update(dict(
            prefix=self.prefix, nbextensions_dir=self.nbextensions_dir,
            overwrite=self.overwrite, symlink=self.symlink))
        try:
            if not self.only_config:
                toggle_install_files(self._toggle_value, **kwargs_files)
            if not self.only_files:
                toggle_install_config(self._toggle_value, **kwargs)
        except NotebookRunningError as err:
            self.log.warn('Error: %s', err)
            self.log.info(
                'To perform actions even while a notebook server is running,'
                'you may use the flag\n--skip-running-check')
            raise


class UninstallContribNbextensionsApp(InstallContribNbextensionsApp):
    """Uninstall jupyter_contrib_nbextensions."""
    _toggle_value = False


class MigrateContribNbextensionsApp(BaseContribNbextensionsInstallApp):
    """
    Migrate config from an old pre-jupyter_contrib_nbextensions install.

    Neatly edits/removes config keys and/or files from installs of
    ipython-contrib/IPython-notebook-extensions.
    """
    name = 'jupyter contrib nbextension migrate'
    description = ('Uninstall any old pre-jupyter_contrib_nbextensions install'
                   ' by removing old config keys and files.')

    def start(self):
        """Perform the App's actions as configured."""
        self.log.info('{} {}'.format(self.name, ' '.join(self.argv)))
        return migrate(logger=self.log)


class ContribNbextensionsApp(BaseContribNbextensionsApp):
    """Main jupyter_contrib_nbextensions application."""

    name = 'jupyter contrib nbextension'
    description = (
        'Install or uninstall all of jupyter_contrib_nbextensions.'
    )
    examples = '\n'.join(['jupyter contrib nbextension ' + t for t in [
        'install   # {}'.format(install.__doc__),
        'uninstall # {}'.format(uninstall.__doc__),
        'migrate   # {}'.format(migrate.__doc__),
    ]])
    subcommands = dict(
        install=(InstallContribNbextensionsApp, install.__doc__),
        uninstall=(UninstallContribNbextensionsApp, uninstall.__doc__),
        migrate=(MigrateContribNbextensionsApp, migrate.__doc__),
    )

    def start(self):
        """Perform the App's functions as configured."""
        super(ContribNbextensionsApp, self).start()

        # The above should have called a subcommand and raised NoStart; if we
        # get here, it didn't, so we should self.log.info a message.
        subcmds = ', '.join(sorted(self.subcommands))
        sys.exit('Please supply at least one subcommand: {}'.format(subcmds))


def jupyter_contrib_core_app_subcommands():
    """Return dict of subcommands for use by jupyter_contrib_core."""
    subcommands = dict(
        nbextension=(ContribNbextensionsApp,
                     ContribNbextensionsApp.description)
    )
    # alias with an 's' as well as without
    subcommands['nbextensions'] = subcommands['nbextension']
    return subcommands


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

main = ContribNbextensionsApp.launch_instance


if __name__ == '__main__':
    main()
